#                       Copyright 2020 Saad Mairaj
# 
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
# 
#        http://www.apache.org/licenses/LICENSE-2.0
# 
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import re
import ast
import pickle as pkl
import tkinter as _TK
import tkmacosx


# Modified Misc._options(...) to make ColorVar work with tkinter
_all_traces_colorvar = {}


def _configure(self, cmd, cnf, kw):
    """Internal function."""
    if kw:
        cnf = _TK._cnfmerge((cnf, kw))
    elif cnf:
        cnf = _TK._cnfmerge(cnf)
    if cnf is None:
        return self._getconfigure(_TK._flatten((self._w, cmd)))
    if isinstance(cnf, str):
        return self._getconfigure1(_TK._flatten((self._w, cmd, '-'+cnf)))

    # -------------------- Added the below block --------------------
    # Add the resources to the list to have ColorVar functionality.
    if isinstance(cmd, tuple) and isinstance(self, _TK.Canvas):
        tags = self.find_withtag(cmd[1])
        cnf_copy = cnf.copy()
        for tag in tags:
            for i in ('activefill', 'activeoutline', 'disabledfill',
                      'disabledoutline', 'fill', 'outline', 'background',
                      'activebackground', 'activeforeground',
                      'disabledbackground', 'disabledforeground',
                      'foreground'):
                if isinstance(cnf_copy.get(i), _TK.Variable):
                    var = cnf_copy[i]
                    cbname = var.trace_add('write', lambda *a,
                                           cls=self, opt=i,
                                           tagId=tag, var=var:
                                           cls.itemconfig(tagId, {opt: var.get()}))
                    if (self, (i, tag)) in _all_traces_colorvar:
                        v, cb = _all_traces_colorvar.get((self, (i, tag)))
                        v.trace_remove('write', cb)
                        _all_traces_colorvar[(self, (i, tag))] = (var, cbname)
                    else:
                        _all_traces_colorvar[(self, (i, tag))] = (var, cbname)
                    cnf[i] = var.get()
    self.tk.call(_TK._flatten((self._w, cmd)) + self._options(cnf))


def _create(self, itemType, args, kw):  # Args: (val, val, ..., cnf={})
    """Internal function."""
    args = _TK._flatten(args)
    cnf = args[-1]
    if isinstance(cnf, (dict, tuple)):
        args = args[:-1]
    else:
        cnf = {}

    # -------------------- Added the below block --------------------
    # Add the resources to the list to have ColorVar functionality.
    ckw = _TK._cnfmerge((cnf, kw))
    var = None
    for i in ('activefill', 'activeoutline', 'disabledfill',
              'disabledoutline', 'fill', 'outline', 'background',
              'activebackground', 'activeforeground',
              'disabledbackground', 'disabledforeground',
              'foreground'):
        if isinstance(ckw.get(i), _TK.Variable):
            var = ckw[i]
            _all_traces_colorvar[(self, (i, None))] = (var, None)
            if i in cnf:
                cnf[i] = var.get()
            elif i in kw:
                kw[i] = var.get()
    # ---------------------------------------------------------------

    tagId = self.tk.getint(self.tk.call(
        self._w, 'create', itemType,
        *(args + self._options(cnf, kw))))

    for key, value in _all_traces_colorvar.copy().items():
        wid, (opt, tag_id) = key
        var, cbname = value
        if tag_id is None and cbname is None:
            cbname = var.trace_add('write', lambda *a,
                                   cls=self, opt=opt,
                                   tagId=tagId, var=var:
                                   cls.itemconfig(tagId, {opt: var.get()}))
            _all_traces_colorvar[(self, (opt, tagId))] = (var, cbname)
            _all_traces_colorvar.pop((self, (opt, None)))
    return tagId


def _options(self, cnf, kw=None):
    """Internal function."""
    if kw:
        cnf = _TK._cnfmerge((cnf, kw))
    else:
        cnf = _TK._cnfmerge(cnf)

    # ----------- Added the below block -------------
    # Add the resources to the list to have ColorVar functionality.
    # It'll work as long as it's not an item of canvas or mark_tag of text.
    for i in ('fg', 'foreground', 'bg', 'background',
              'activebackground', 'activeforeground', 'disabledforeground',
              'highlightbackground', 'highlightcolor', 'selectforeground',
              'readonlybackground', 'selectbackground', 'insertbackground',
              'disabledbackground', 'fill'):
        if isinstance(cnf.get(i), _TK.Variable):
            var = cnf[i]
            if not isinstance(self, tkmacosx.Button) and cnf.get('fill'):
                continue
            elif isinstance(self, tkmacosx.Button) and cnf.get('fill'):
                i = 'fg'
            cbname = var.trace_add('write', lambda *a, i=i, var=var,
                                   cls=self: cls.config({i: var.get()}))
            if (self, i) in _all_traces_colorvar:
                v, cb = _all_traces_colorvar.get((self, i))
                v.trace_remove('write', cb)
                _all_traces_colorvar[(self, i)] = (var, cbname)
            else:
                _all_traces_colorvar[(self, i)] = (var, cbname)
            if isinstance(self, tkmacosx.Button) and cnf.get('fill'):
                i = 'fill'
            cnf[i] = var.get()
    # -----------------------------------------------
    res = ()
    for k, v in cnf.items():
        if v is not None:
            if k[-1] == '_':
                k = k[:-1]
            if callable(v):
                v = self._register(v)
            elif isinstance(v, (tuple, list)):
                nv = []
                for item in v:
                    if isinstance(item, int):
                        nv.append(str(item))
                    elif isinstance(item, str):
                        nv.append(_TK._stringify(item))
                    else:
                        break
                else:
                    v = ' '.join(nv)
            res = res + ('-'+k, v)
    return res


_TK.Misc._options = _options
_TK.Canvas._create = _create
_TK.Misc._configure = _configure


class ColorVar(_TK.Variable):
    """Value holder for HEX color. Default is white"""

    _default = "white"
    _rgbstring = re.compile(r'#[a-fA-F0-9]{3}(?:[a-fA-F0-9]{3})?$')

    def __init__(self, master=None, value=None, name=None):
        """Construct a color variable. (bg, fg, ..)

        MASTER can be given as master widget.
        VALUE is an optional value (defaults to "")
        NAME is an optional Tcl name (defaults to PY_VARnum).

        If NAME matches an existing variable and VALUE is omitted
        then the existing value is retained.
        """
        super(ColorVar, self).__init__(master, value, name)

    def set(self, value=''):
        """Set the variable to VALUE."""
        if value.startswith('#'):
            if not bool(self._rgbstring.match(value)):
                raise ValueError('{} is not a vaild HEX.'.format(value))
        elif isinstance(value, str):
            try:
                r, g, b = self._root.winfo_rgb(value)
                c = (r/257, g/257, b/257)
                value = '#%02x%02x%02x' % (int(c[0]), int(c[1]), int(c[2]))
            except:
                raise ValueError(
                    'Could not find right HEX for "{}".'.format(value))
        return self._tk.globalsetvar(self._name, value)
    initialize = set

    def get(self):
        """Return value of variable color."""
        value = self._tk.globalgetvar(self._name)
        if isinstance(value, str):
            return value
        return str(value)


class DictVar(_TK.Variable):
    """
    #### Value holder for Dictionaries.
    Get a specific value by getting the key from this \
    `get(self, key=None, d=None)` method if exists in the dictionary. \n
    if `key=None` it will returrn the complete dictionory.
    """
    _default = {}

    def __init__(self, master=None, value=None, name=None):
        """Construct a string variable.

        MASTER can be given as master widget.
        VALUE is an optional value (defaults to {})
        NAME is an optional Tcl name (defaults to PY_VARnum).

        If NAME matches an existing variable and VALUE is omitted
        then the existing value is retained.
        """
        _TK.Variable.__init__(self, master, value, name)

    def get(self, key=None, d=None):
        """Return value of variable as string."""
        value = self._tk.globalgetvar(self._name)
        if not isinstance(value, dict):
            value = ast.literal_eval(value)
        if key:
            return value.get(key, d)
        else:
            return value


def SaveVar(var: _TK.Variable, master=None, value=None, name=None, filename='data.pkl') \
        -> (_TK.Variable):
    """Save tkinter variable data in a pickle file and load the 
    same value when the program is executed next time. 

    #### If the content of the file changes, it might not load correct values \
        to the assigned variables. To avoid this issue use `name` to \
        refer to the exact assigned values later.

    ### Args:
    - `var`: Give the `tkinter.Variable` class like (`tk.StringVar`, `tk.IntVar`).
    - `master`: Parent widget.
    - `value`: Set value.
    - `name`: Set a name to group variables or to refer to assigned value when loaded.
    - `filename`: Set the name of the save file. (To make the file invisible in the \
            directory start the name of the file with "." like ".cache-savevar")

    ### Return:
    - returns the tk.Variable instance passed to `var` argument.

    ### Example:
        root = tk.Tk()
        var1 = SaveVar(tk.StringVar,'Enter Username','Var1','.cache-savevar')
        var2 = SaveVar(tk.StringVar,'Enter Password','Var2','.cache-savevar')
        tk.Entry(root,textvariable=var1).pack()
        tk.Entry(root,textvariable=var2).pack()
        root.mainloop()"""

    def update_val(*args):
        """Internal function for updating the value for variable"""
        try:    # try/except , if the file doesn't exists.
            open1 = open(filename, 'rb')
            tmpdict = pkl.load(open1)  # load saved dictionary data.
            # Block of code to check for the right value.
            if tmpdict.get(str(var)):
                old, default = tmpdict.get(str(var))
                new = var.get()
                if new != default:
                    var.set(new)
                elif new == default and not startup[0]:
                    var.set(default)
                else:
                    var.set(old)
            tmpdict.update({str(var): (var.get(), defaultval)})
            open1.close()
        except Exception as e:
            tmpdict = {}
            tmpdict[str(var)] = (var.get(), defaultval)

        open2 = open(filename, 'wb')
        pkl.dump(tmpdict, open2)
        startup[0] = False
        open2.close()

    startup = [True]
    if not(filename.endswith('.pickle') or filename.endswith('.pkl')) \
            and not filename.startswith('.'):
        filename = filename+'.pkl'
    var = var(master=master, value=value, name=name)
    defaultval = var.get()  # get a default value of the variable
    update_val()
    for mode, cbname in (var.trace_info()):
        if mode[0] == 'write' and update_val.__name__ in cbname:
            try:
                var.trace_remove('write', cbname)
            except:
                pass
    res = var.trace_add('write',  update_val)
    return var


# ------------------ Testing ------------------

def demo_colorvar():
    import tkmacosx.colors as colors
    root = _TK.Tk()
    root.geometry('100x100')
    color = ColorVar()
    color_list = list(colors.OrderedHex)
    L = _TK.Label(root, textvariable=color, bg=color)
    L.place(relx=0.5, rely=0.5, anchor='center')

    def change_color(c=0):
        if root.winfo_exists():
            if c >= len(color_list):
                c = 0
            color.set(color_list[c])
            root.after(100, change_color, c+1)

    change_color()
    root.mainloop()


def demo_savevar():
    root = _TK.Tk()
    var1 = SaveVar(_TK.StringVar, root, 'Enter Username',
                   'Var1', '.cache-savevar')
    var2 = SaveVar(_TK.StringVar, root, 'Enter Password',
                   'Var2', '.cache-savevar')
    _TK.Entry(root, textvariable=var1).pack()
    _TK.Entry(root, textvariable=var2).pack()
    root.mainloop()


if __name__ == "__main__":
    demo_colorvar()
    demo_savevar()



#                       Copyright 2020 Saad Mairaj
# 
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
# 
#        http://www.apache.org/licenses/LICENSE-2.0
# 
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import tkinter as _TK
from tkmacosx.basewidget import _Frame, _Canvas, Widget


class SFrame(_Frame):
    """### Scrollable Frame Widget.    
    (Only supports vertical scrolling)

    ### Agrs:
    Sames as tkinter Frame. These are some extra resources.
    1. `scrollbarwidth`: Set the width of scrollbar.
    2. `mousewheel`: Set mousewheel scrolling.

    ### How to use?
    Use it like a normal frame.

    ### Example:

        root = Tk()
        frame = SFrame(root, bg='pink')
        frame.pack()

        for i in range(100):
            Button(frame, text='Button %s'%i).pack()

        root.mainloop()
    """

    def __init__(self, master=None, **kw):
        self.scrollbarwidth = kw.pop('scrollbarwidth', 10)
        self.mousewheel = kw.pop('mousewheel', 1)
        self._canvas = _TK.Canvas(master=master, highlightthickness=0, width=kw.pop('width', 250),
                                  height=kw.pop('height', 250))
        _Frame.__init__(self, self._canvas, **kw)
        self.yScroll = _TK.Scrollbar(
            self._canvas, orient='vertical', width=self.scrollbarwidth)
        self.yScroll.place(relx=1, rely=0, anchor='ne', relheight=1)
        self.yScroll.configure(command=self._canvas.yview)
        self._canvas.configure(yscrollcommand=self.yScroll.set)
        self._canvas.create_window(0, 0, anchor='nw', tags="window", window=self,
                                   width=self._canvas.winfo_reqwidth()-self.yScroll.winfo_reqwidth())
        self._canvas.bind("<Configure>", self._configure_height, add="+")
        self.bind_class(self, "<Configure>", self._configure_window, add="+")
        self._mouse_scrolling()
        self._geometryManager()

    def _mouse_scrolling(self):
        if self.mousewheel:
            self.bind_class(self, '<Enter>', lambda _:
                            self.bind_all('<MouseWheel>', self._on_mouse_scroll))
            self.bind_class(self, '<Leave>', lambda _:
                            self.unbind_all('<MouseWheel>'))
        else:
            self.unbind_class(self, '<Enter>')
            self.unbind_class(self, '<Leave>')

    def _on_mouse_scroll(self, evt):
        if evt.state == 0:
            self._canvas.yview_scroll(-1*(evt.delta), 'units')

    def _configure_height(self, evt):
        if self.winfo_height() < self._canvas.winfo_height():
            self._canvas.itemconfig(
                "window", height=self._canvas.winfo_height(), anchor='nw')
        self._canvas.itemconfig(
            'window', width=self._canvas.winfo_width()-self.yScroll.winfo_width())

    def _configure_window(self, evt):
        # this will update the position of scrollbar when scrolled from mousewheel.
        # fixes some bugs
        # makes scrolling more smoother
        self.update()
        self._canvas.configure(scrollregion=self._canvas.bbox('all'))

    def _geometryManager(self):
        """Internal funtion."""
        # Can be done in a loop with setatr but then it underlines with red when these
        # attributes are called as they're created at runtime.
        self.grid = self.grid_configure = self._canvas.grid_configure
        self.grid_forget = self._canvas.grid_forget
        self.grid_info = self._canvas.grid_info
        self.grid_remove = self._canvas.grid_remove
        self.pack = self.pack_configure = self._canvas.pack_configure
        self.pack_forget = self._canvas.pack_forget
        self.pack_info = self._canvas.pack_info
        self.place = self.place_configure = self._canvas.place_configure
        self.place_forget = self._canvas.place_forget
        self.place_info = self._canvas.place_info

    def configure(self, cnf=None, **kw):
        kw = _TK._cnfmerge((cnf, kw))
        self._canvas['width'] = kw.pop('width', self._canvas['width'])
        self._canvas['height'] = kw.pop('height', self._canvas['height'])
        self.yScroll['width'] = kw.pop('scrollbarwidth', self.yScroll['width'])
        self.mousewheel = kw.pop('mousewheel', self.mousewheel)
        self._mouse_scrolling()
        return super().configure(**kw)

    def cget(self, key):
        if key in ('scrollbarwidth', 'mousewheel'):
            return self.__dict__.get(key)
        return super().cget(key)
    config = configure
    __getitem__ = cget


class Button(Widget):
    """ Button for macos, supports almost all the features of tkinter button,
    - Looks very similar to ttk Button.
    - There are few extra features as compared to default Tkinter Button:
    - To check the list of all the resources. To get an overview about
        the allowed keyword arguments call the method keys 
            print(Button().keys())

    ### Examples:
        import tkinter as tk
        import tkmacosx as tkm
        root = tk.Tk()
        root.geometry('200x200')
        tkm.Button(root, text='Mac OSX', bg='lightblue', fg='yellow').pack()
        tk.Button(root, text='Mac OSX', bg='lightblue', fg='yellow').pack()
        ttk.Button(root, text='Mac OSX').pack()
        root.mainloop()

    ### Get a cool gradient effect in activebackground color.
        import tkinter as tk
        import tkmacosx as tkm
        root = tk.Tk()
        root.geometry('200x200')
        tkm.Button(root, text='Press Me!!', activebackground=('pink','blue') ).pack()
        tkm.Button(root, text='Press Me!!', activebackground=('yellow','green') ).pack()
        tkm.Button(root, text='Press Me!!', activebackground=('red','blue') ).pack()
        root.mainloop()"""

    # all the instance of class Button will be stored in _button list.

    def __init__(self, master=None, cnf={}, **kw):
        super(Button, self).__init__(master=master, cnf=cnf, **kw)

    def invoke(self):
        """Invoke the command associated with the button.

        The return value is the return value from the command,
        or an empty string if there is no command associated with
        the button. This command is ignored if the button's state
        is disabled.
        """
        if self['state'] == 'normal':
            return self.cnf['command']() if self.cnf.get('command') else None


class Marquee(_TK.Canvas):
    """Use `Marquee` for creating scrolling text which moves from 
    right to left only if the text does not fit completely.

    ### Args:
    - `text`: Give a string to display.
    - `font`: Font of the text.
    - `fg`: Set foreground color of the text.
    - `fps`: Set fps(frames per seconds).
    - `left_margin`: Set left margin to make text move further to right before reset.
    - `initial_delay`: Delay before text start to move.
    - `end_delay`: Delay before text reset.
    - `smoothness`: Set the smoothness of the animation.

    ### Example: 
        root=tk.Tk()
        marquee = Marquee(root, text='This text will move from right to left if does not fit the window.')
        marquee.pack()
        root.mainloop()"""

    def __init__(self, master=None, cnf={}, **kw):
        kw = _TK._cnfmerge((cnf, kw))
        self.cnf = dict(
            text=kw.pop('text', ''),
            font=kw.pop('font', None),
            fg=kw.pop('fg', 'black') if kw.get(
                'fg') else kw.pop('foreground', 'black'),
            fps=kw.pop('fps', 30),
            left_margin=kw.pop('left_margin', 10),
            initial_delay=kw.pop('initial_delay', 1000),
            end_delay=kw.pop('end_delay', 1000),
            smoothness=kw.pop('smoothness', 1),  # 1 <= smooth < 1
        )
        kw['height'] = kw.get('height', 24)
        kw['highlightthickness'] = kw.get('highlightthickness', 0)
        super().__init__(master=master, **kw)
        self.create_text(3, 1, anchor='w', tag='text', text=self.cnf.get('text'),
                         font=self.cnf.get('font'), fill=self.cnf.get('fg'))
        self.bind_class(self, '<Configure>', self.check, '+')
        self.after_id = ' '

    def check(self, evt=None):
        "Sets the text properly in the frame."
        self.coords('text', 3, self.winfo_height()/2)
        text_width = self.bbox('text')[2]   # TEXT WIDTH
        frame_width = self.winfo_width()    # FRAME WIDTH
        if text_width + 1 < frame_width:
            self.after_cancel(self.after_id)
            self.coords('text', 3, self.winfo_height()/2)  # RESETS TEXT
            self.after_id = ' '
        elif self.after_id == ' ':
            delay = self.cnf.get('initial_delay')  # INITITAL DEALY
            self.after_id = self.after(delay, self.animate)

    def animate(self, evt=None):
        "Process text and move text."
        text_width = self.bbox('text')[2]   # TEXT WIDTH
        frame_width = self.winfo_width()    # FRAME WIDTH
        delay = int(self.cnf.get('smoothness')*1000 / self.cnf.get('fps'))
        if text_width + 1 + self.cnf.get('left_margin') < frame_width:
            self.after(self.cnf.get('end_delay'), self.coords,
                       'text', 3, self.winfo_height()/2)  # RESETS TEXT
            delay = self.cnf.get('initial_delay') + self.cnf.get('end_delay') # INITITAL DEALY
        else:
            # MOVE -1 PIXEL EVERYTIME
            self.move('text', -self.cnf.get('smoothness'), 0)
        self.after_id = self.after(delay, self.animate)

    def configure(self, cnf={}, **kw):
        kw = _TK._cnfmerge((cnf, kw))
        self.cnf = dict(
            text=kw.pop('text', self.cnf.get('text')),
            font=kw.pop('font', self.cnf.get('font')),
            fg=kw.pop('fg', self.cnf.get('fg')) if kw.get('fg')
            else kw.pop('foreground', self.cnf.get('foreground')),
            fps=kw.pop('fps', self.cnf.get('fps')),
            left_margin=kw.pop('left_margin', self.cnf.get('left_margin')),
            initial_delay=kw.pop(
                'initial_delay', self.cnf.get('initial_delay')),
            end_delay=kw.pop('end_delay', self.cnf.get('end_delay')),
            smoothness=kw.pop('smoothness', self.cnf.get('smoothness')),
        )
        self.itemconfig('text', text=self.cnf.get('text'),
                        font=self.cnf.get('font'), fill=self.cnf.get('fg'))
        return super().configure(**kw)

    def cget(self, key):
        if key in self.cnf.keys():
            return self.cnf['key']
        return super().cget(key)
    __getitem__ = cget
    config = configure

# ------------------ Testing ------------------


def demo_button():
    from tkinter import ttk
    root = _TK.Tk()
    root.title('Mac OSX Button Demo')
    page_color = '#FFFFC6'
    root.geometry('300x400')
    root['bg'] = page_color
    _TK.Label(root, text='Can you tell the difference?', font=(
        '', 16, 'bold'), bg=page_color).pack(pady=(20, 5))
    Button(root, text='Press Me', disabledbackground='red').pack(pady=(0, 5))
    ttk.Button(root, text="Press Me").pack()
    _TK.Label(root, text='Change Background Color', font=(
        '', 16, 'bold'), bg=page_color).pack(pady=(20, 5))
    Button(root, text='Press Me', bg='pink',
           activebackground=('pink', 'blue')).pack()
    _TK.Label(root, text='Blend In', font=('', 16, 'bold'),
              bg=page_color).pack(pady=(20, 5))
    Button(root, text='Press Me', bg='yellow', activebackground=(
        'orange', 'lime'), borderless=1).pack()
    _TK.Label(root, text='Change bordercolor', font=(
        '', 16, 'bold'), bg=page_color).pack(pady=(20, 5))
    Button(root, text='Press Me', bg='red', fg='white',
           activebackground=('red', 'blue'), bordercolor='blue').pack()
    root.mainloop()


def demo_sframe():
    root = _TK.Tk()
    frame = SFrame(root, bg='pink')
    frame.pack()
    for i in range(50):
        Button(frame, text='Button %s' % i, borderless=1).pack()
    root.mainloop()


def demo_marquee():
    root = _TK.Tk()
    marquee = Marquee(
        root, text='This text will move from right to left if does not fit the window.')
    marquee.pack()
    root.mainloop()


if __name__ == "__main__":
    demo_sframe()
    demo_button()
    demo_marquee()



#                       Copyright 2020 Saad Mairaj
# 
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
# 
#        http://www.apache.org/licenses/LICENSE-2.0
# 
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from tkinter import ttk
import tkinter as _TK
import re
import subprocess

try: 
    import colour as C
except Exception as e:
    import sys, os
    import tkinter.messagebox as ms
    Error_win = _TK.Tk()
    Error_win.withdraw()
    message = """
    Module "tkmacosx" is dependent on 
    "colour" Module. 

    Do you want to install it with pip now?
    Or install it manually later.
    
    Link to the colour github:
    https://github.com/vaab/colour"""
    if ms._show(e, message, 'warning', 'yesno')=='yes':
        Error_win.destroy()
        os.system('pip install colour')
        import colour as C
    else: 
        sys.exit(0)


def check_appearence(cmd='defaults read -g AppleInterfaceStyle'):
    """### Checks DARK/LIGHT mode of macos. Returns Boolean.
    #### Args:
    - `cmd`: Give commands. Like to check DARK/LIGHT mode the command is `'defaults read -g AppleInterfaceStyle'` .
    """
    out, err = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
            universal_newlines=True, shell=True).communicate()
    if out: return True
    elif err: return False


def get_shade(color, shade: float, mode='auto'):
    """### Darken or Lghten a shade of A HEX color.
    #### Args:
    1. `color`: Give a color as either HEX or name of the color.
    2. `shade`: The amount of change required. Takes float. eg: shade=0.225.
    3. `mode`: 
        - `'-'` for darker shade.
        - `'+'` for lighter shade.
        - `'auto-110'` automatically decide lighter or darker. where 110 is the intensity.
    
    return hexcode"""
    if isinstance( color, str ):
        if color.startswith('#'):
            Hex = color.lstrip('#')
            color = tuple( int(Hex[i:i+2], 16) for i in (0, 2, 4) )
        else:  
            tmp = _TK.Frame()
            r,g,b = tmp.winfo_rgb(color)
            tmp.destroy()
            color = (r/257, g/257, b/257)
    if 'auto' in mode:
        intensity = 110.0 if len(mode)<=4 else float(mode.split('-')[1])
        mode = '-' if float(color[0]*0.299 + color[1]*0.587 \
                        + color[2]*0.114) > intensity else '+'
    if mode == '+':
        R = float(color[0]*(1-shade) + shade*255) \
                if float(color[0]*(1-shade) + shade*255) > 0 else 0.0
        G = float(color[1]*(1-shade) + shade*255) \
                if float(color[1]*(1-shade) + shade*255) > 0 else 0.0
        B = float(color[2]*(1-shade) + shade*255) \
                if float(color[2]*(1-shade) + shade*255) > 0 else 0.0
    elif mode == '-':
        R = float(color[0]*(1-shade) - shade*255) \
                if float(color[0]*(1-shade) - shade*255) > 0 else 0.0
        G = float(color[1]*(1-shade) - shade*255) \
                if float(color[1]*(1-shade) - shade*255) > 0 else 0.0
        B = float(color[2]*(1-shade) - shade*255) \
                if float(color[2]*(1-shade) - shade*255) > 0 else 0.0
    else: raise ValueError ('Invalid mode "{}"'. format(mode))
    return '#%02x%02x%02x' % (int(R),int(G),int(B))  


class _Frame(_TK.BaseWidget):
    """Don't use this Frame widget. The widget has no geometry manager.
    
    It is used in SFrame widget to support screen."""
    def __init__(self, master=None, cnf={}, **kw):
        cnf = _TK._cnfmerge((cnf, kw))
        _TK.BaseWidget.__init__(self, master, 'frame', cnf, {})


class _Canvas(_TK.Widget):
    """Internal Class especially for Button widget"""
    def __init__(self, master=None, cnf={}, **kw):
        super(_Canvas, self).__init__(master, 'canvas', cnf, kw)
    
    def find(self, *args):
        """Internal function."""
        return self._getints(
            self.tk.call((self._w, 'find') + args)) or ()
    def bbox(self, *args):
        """Return a tuple of X1,Y1,X2,Y2 coordinates for a rectangle
        which encloses all items with tags specified as arguments."""
        return self._getints(
            self.tk.call((self._w, 'bbox') + args)) or None
    def coords(self, *args):
        """Return a list of coordinates for the item given in ARGS."""
        return [self.tk.getdouble(x) for x in
                           self.tk.splitlist(
                   self.tk.call((self._w, 'coords') + args))]
    def _create(self, itemType, args, kw): # Args: (val, val, ..., cnf={})
        """Internal function."""
        args = _TK._flatten(args)
        cnf = args[-1]
        if isinstance(cnf, (dict, tuple)):
            args = args[:-1]
        else:
            cnf = {}
        return self.tk.getint(self.tk.call(
            self._w, 'create', itemType,
            *(args + self._options(cnf, kw))))
    def _arc(self, *args, **kw):
        """Create arc shaped region with coordinates x1,y1,x2,y2."""
        return self._create('arc', args, kw)
    def _bitmap(self, *args, **kw):
        """Create bitmap with coordinates x1,y1."""
        return self._create('bitmap', args, kw)
    def _image(self, *args, **kw):
        """Create image item with coordinates x1,y1."""
        return self._create('image', args, kw)
    def _line(self, *args, **kw):
        """Create line with coordinates x1,y1,...,xn,yn."""
        return self._create('line', args, kw)
    def _text(self, *args, **kw):
        """Create text with coordinates x1,y1."""
        return self._create('text', args, kw)
    def _rectangle(self, *args, **kw):
        """Create rectangle with coordinates x1,y1,x2,y2."""
        return self._create('rectangle', args, kw)
    def delete(self, *args):
        """Delete items identified by all tag or ids contained in ARGS."""
        self.tk.call((self._w, 'delete') + args)
    def itemcget(self, tagOrId, option):
        """Return the resource value for an OPTION for item TAGORID."""
        return self.tk.call(
            (self._w, 'itemcget') + (tagOrId, '-'+option))
    def itemconfigure(self, tagOrId, cnf=None, **kw):
        """Configure resources of an item TAGORID.

        The values for resources are specified as keyword
        arguments. To get an overview about
        the allowed keyword arguments call the method without arguments.
        """
        return self._configure(('itemconfigure', tagOrId), cnf, kw)
    itemconfig = itemconfigure
    def tag_lower(self, *args):
        """Lower an item TAGORID given in ARGS
        (optional below another item)."""
        self.tk.call((self._w, 'lower') + args)
    lower = tag_lower
    def tag_raise(self, *args):
        """Raise an item TAGORID given in ARGS
        (optional above another item)."""
        self.tk.call((self._w, 'raise') + args)
    lift = tkraise = tag_raise
    def rounded_rect(self, x, y, w, h, c, tag1=None, tag2=None, **kw):
        'Internal function.'
        # Need fix to just have one tag to change the background color
        kw['extent'] = 90
        kw['style'] = 'arc'
        kw['outline'] = kw.pop('fill', 'black')
        if tag1: kw['tag'] = tag1
        self._arc(x,   y,   x+2*c,   y+2*c,   start= 90, **kw)
        self._arc(x+w-2*c, y+h-2*c, x+w, y+h, start=270, **kw)
        self._arc(x+w-2*c, y,   x+w, y+2*c,   start=  0, **kw)
        self._arc(x,   y+h-2*c, x+2*c,   y+h, start=180, **kw)
        kw.pop('extent', None)
        kw.pop('style', None)
        kw['fill'] = kw.pop('outline', None)
        if tag2: kw['tag'] = tag2
        self._line(x+c, y,   x+w-c, y    , **kw)
        self._line(x+c, y+h, x+w-c, y+h  , **kw)
        self._line(x,   y+c, x,     y+h-c, **kw)
        self._line(x+w, y+c, x+w,   y+h-c, **kw)
    def _rounded(self, x1, y1, x2, y2, r,**kw):
        self._arc(x1,  y1,  x1+r,   y1+r, start= 90, extent=90, style='pieslice', outline="", **kw)
        self._arc(x2-r-1, y1, x2-1, y1+r, start=  0, extent=90, style='pieslice', outline="", **kw)
        self._arc(x1, y2-r-1, x1+r, y2-1, start=180, extent=90, style='pieslice', outline="", **kw)
        self._arc(x2-r, y2-r, x2-1, y2-1, start=270, extent=90, style='pieslice', outline="", **kw)
        self._rectangle(x1+r/2, y1, x2-r/2, y2, width=0, **kw)
        self._rectangle(x1, y1+r/2, x2, y2-r/2, width=0, **kw)


class Widget(_Canvas):
    """Internal class used for tkinter macos Buttton"""

    _buttons = []  # list of all buttons
    _features =  [  'activebackground', 'activeforeground', 'activeimage', 'activebitmap', 'anchor', 'bitmap', 
                    'borderwidth', 'bd', 'bordercolor', 'borderless', 'command', 'compound', 'disabledforeground', 
                    'disabledbackground', 'fg', 'font', 'foreground', 'height', 'image', 'overrelief', 'padx', 
                    'pady', 'repeatdelay', 'repeatinterval', 'text', 'textvariable', 'underline', 'width' ]

    def __init__(self, master=None, cnf={}, **kw):
        kw = _TK._cnfmerge( (cnf, kw) )
        kw = { k:v for k,v in kw.items() if v is not None }
        self.stopped = None
        self.cnf = {}
        for i in kw.copy().keys():
            if i in self._features: self.cnf[i] = kw.pop(i, None)

        self.cnf['fg'] = self.cnf.get('fg') if self.cnf.get('fg', None) else self.cnf.get('foreground','black')
        self.cnf['bd'] = self.cnf.get('bd') if self.cnf.get('bd', None) else self.cnf.get('borderwidth',6)
        self.cnf['borderless'] = self.cnf.get('borderless', False)
        self.cnf['disabledforeground'] = self.cnf.get('disabledforeground', 'grey')
        if self.cnf.get('textvariable') is not None: self.cnf['text'] = self.cnf['textvariable'].get()
        self.cnf['anchor'] = self.cnf.get('anchor', 'center')

        kw['takefocus'] = kw.get('takefocus', 1)
        kw['highlightthickness'] = kw.get('highlightthickness', 0)
        kw['width'] = kw.get('width', 87)
        kw['height'] = kw.get('height', 24)

        super(Widget, self).__init__(master=master, **kw)
        self._buttons.append(self)
        self._size = (self.winfo_width(), self.winfo_height())
        if self.cnf.get('text'): self._text(0,0,text=None, tag='_txt')
        if self.cnf.get('image'): self._image(0,0,image=None, tag='_img')
        elif self.cnf.get('bitmap'): self._bitmap(0,0,image=None, tag='_bit')
        self.bind_class('button_release', '<ButtonRelease-1>', self.on_release, '+')
        self.bind_class('button_press', '<Button-1>', self.on_press, '+' )
        self.bind_class('set_size', '<Configure>', self._set_size, '+')
        self.original_bg = self['bg']

        #  Focus in and out effect 
        main_win =  self.winfo_toplevel()
        def _chngIn(evt):
            if self.focus_get() is None:
                color = get_shade(self['bg'], 0.04, 'auto-120')
                self.itemconfig('_border1', outline=color)
                self.itemconfig('_border2', fill=color)
            if self.focus_get() and get_shade(self['bg'], 0.04, 'auto-120') == self.itemcget('_border2','fill'):
                color = get_shade(self['bg'], 0.1, 'auto-120')
                self.itemconfig('_border1', outline=color)
                self.itemconfig('_border2', fill=color)
        main_win.bind_class(main_win,'<FocusIn>', _chngIn, '+')
        main_win.bind_class(main_win,'<FocusOut>', _chngIn, '+')
        self._getconfigure2(self.cnf)

    def _set_size(self, *ags):
        """Internal function. This will resize everything that is in the button"""
        if ags[0].width == self._size[0] and ags[0].height == self._size[1]: return
        self._size = (ags[0].width, ags[0].height)
        self.delete('_bd_color1')
        self.delete('_bd_color2')
        self.delete('_border1')
        self.delete('_border2')
        self.delete('_tf')
        # Need fix (laggy on resizing) ----> workaround: cancel if still resizing 
        if self.stopped: self.after_cancel(self.stopped)
        self.stopped = self.after(50, lambda: self.on_press_color(tag='_activebg', 
            width=ags[0].width, height=ags[0].height,color=self.cnf.get('activebackground'))  )
        self.rounded_rect(0, -1, ags[0].width, ags[0].height+3, 6, width=self.cnf.get('bd',6), fill=self.cnf.get('bordercolor', 
                get_shade(self['bg'], 0.04, 'auto-120')), tag1='_bd_color1', tag2='_bd_color2')
        self.rounded_rect(2, 2, ags[0].width-5, ags[0].height-4, 3, width=1, fill=get_shade(self['bg'], 0.1, 'auto-120'), 
                tag1='_border1', tag2='_border2')
        self.rounded_rect(2, 2, ags[0].width-4, ags[0].height-3, 4, width=2, fill='#81b3f4', tag='_tf', state='hidden')
        self.coords('_txt', ags[0].width/2, ags[0].height/2)
        self.coords('_img', ags[0].width/2, ags[0].height/2)
        self.coords('_bit', ags[0].width/2, ags[0].height/2)
        self._compound(self.cnf.get('compound'), width=ags[0].width, height=ags[0].height)
        self.tag_raise('_txt')
        self.tag_raise('_img')
        self.tag_raise('_bit')
        self.tag_raise('_tf')
        self.after(50, self._getconfigure2)
        # self.master.focus()
        ## NEED TESTS:- 
        ## if this is really needed, then change master with root 
        ## window as this set focus to entry widget if the master is in an entry widget.

    def on_press(self, *ags):
        ''' Internal function. When button is pressed <Button-1>'''
        self._rpin = None
        self._rpinloop = True

        def cmd(*a):
            self.cnf['command']() if self.cnf.get('command') else None
            self.unbind_class('button_command', '<ButtonRelease-1>')
            if self.cnf.get('repeatdelay', 0) and self.cnf.get('repeatinterval', 0) and self._rpinloop:
                self._rpin = self.after( self.cnf.get('repeatinterval', 0), cmd )
        def on_enter(*a):
            self.itemconfig('_activebg', state='normal')
            self.itemconfig('_border1', state='hidden')
            self.itemconfig('_border2', state='hidden')
            if self.cnf.get('repeatdelay',0) and self.cnf.get('repeatinterval', 0):
                self._rpinloop = True
                cmd()
            self.bind_class('button_command', '<ButtonRelease-1>', 
                lambda *a: self.after(0, cmd), '+')
        def on_leave(*a):
            self.itemconfig('_activebg', state='hidden')
            self.itemconfig('_border1', state='normal')
            self.itemconfig('_border2', state='normal')
            if self.cnf.get('repeatdelay',0) and self.cnf.get('repeatinterval', 1):
                self._rpinloop = False
                self.after_cancel(self._rpin)
            self.unbind_class('button_command', '<ButtonRelease-1>')

        if self['state'] == 'normal':
            self.focus_set()
            self.itemconfig('_activebg', state='normal')
            self.itemconfig('_border1', state='hidden')
            self.itemconfig('_border2', state='hidden')
            self.bind_class('on_press_enter', '<Enter>', on_enter, '+')
            self.bind_class('on_press_leave', '<Leave>', on_leave, '+')
            if self.cnf.get('repeatdelay',0) and self.cnf.get('repeatinterval', 1):
                self._rpin = self.after(self.cnf.get('repeatdelay',0), cmd)
            self.bind_class('button_command', '<ButtonRelease-1>', cmd, '+')
            
    def on_release(self, *ags):
        '''Internal function. When button is released <ButtonRelease-1>'''
        if self['state'] == 'normal':
            self.itemconfig('_activebg', state='hidden')
            self.itemconfig('_border1', state='normal')
            self.itemconfig('_border2', state='normal')
            self.unbind_class('on_press_enter', '<Enter>')
            self.unbind_class('on_press_leave', '<Leave>')
            self.unbind_class('button_command', '<ButtonRelease-1>')
            self._rpinloop = False
            if self._rpin: self.after_cancel(self._rpin) 

    def _configure(self, cmd, cnf=None, kw=None):
        'Internal function to configure the inherited class'
        return super()._configure(cmd, cnf, kw)

    def configure(self, cnf=None, **kw):
        """Configure resources of a widget.

        The values for resources are specified as keyword
        arguments. To get an overview about
        the allowed keyword arguments call the method keys.

        Returns tuple of 2 configuration settings info, if any
        """
        kw = _TK._cnfmerge( (cnf, kw) )
        cnf = {}
        for i in list(kw):
            if i in self._features: 
                cnf[i] = kw.pop(i,None)
        r1 = super().configure(**kw)
        if kw.get('bg') or kw.get('background'):
            self.original_bg = self['bg']
        self.after(10, self._getconfigure2, cnf)
        if r1 is not None:
            r1.update(self.cnf)
        return r1
    config = configure

    def cget(self, key):
        """Return the resource value for a KEY given as string."""
        if key in self._features: return self.cnf[key]
        else: return super().cget(key)
    __getitem__ = cget

    def _getconfigure2(self, cnf={}, **kw):
        """Internal Function.
        This function configure all the resouces of the Widget and save it to the class dict."""
        kw = _TK._cnfmerge( (cnf, kw) )
        kw = { k:v for k,v in kw.items() if v is not None }        
        self.cnf.update(kw)
        self.cnf = { k:v for k, v in self.cnf.items() if v is not None }
        self.cnf['fg'] = self.cnf.get('fg') if self.cnf.get('fg', None) else self.cnf.get('foreground','black')
        self.cnf['bd'] = self.cnf.get('bd') if self.cnf.get('bd', None) else self.cnf.get('borderwidth',6)
        if self.cnf.get('textvariable') is not None: self.cnf['text'] = self.cnf['textvariable'].get()

        if self['state'] == 'disabled':
            super().configure(bg=self.cnf.get('disabledbackground'))
        elif self['state'] == 'normal':
            super().configure(bg=self.original_bg)

        self._rel = self['relief']
        if self.cnf.get('overrelief') is not None:
            self._rel = self['relief']
            self.bind_class("overrelief", "<Enter>", lambda _: self.config(relief=kw['overrelief']), '+')
            self.bind_class("overrelief", "<Leave>", lambda _: self.config(relief=self._rel), '+')
        else:
            self.unbind_class('overrelief', '<Enter>')
            self.unbind_class('overrelief', '<Leave>')
            self._configure('configure', { 'relief': self._rel }, kw=None)
        
        if kw.get('activebackground') is not None:
            self.on_press_color(tag='_activebg', width=self.winfo_reqwidth(), 
                height=self.winfo_reqheight(), color=self.cnf.get('activebackground'))
            self.tag_lower('_activebg')
        elif kw.get('activebackground') is '': 
            self.cnf.pop('activebackground', None)
            self.on_press_color(tag='_activebg', width=self.winfo_reqwidth(), 
                height=self.winfo_reqheight(), color=None)
        
        if kw.get('activeforeground') is not None:
            self.bind_class('active_fg','<Enter>', lambda _:self.itemconfig('_txt',fill=kw['activeforeground'] ))
            self.bind_class('active_fg','<Leave>', lambda _:self.itemconfig('_txt',fill=self.cnf.get('fg','black')) )
        elif kw.get('activeforeground') is '':
            self.unbind_class('active_fg','<Enter>')
            self.unbind_class('active_fg','<Leave>')
            self.itemconfig('_txt',fill=self.cnf.get('fg','black'))
        
        for set_size_if in ('text', 'font', 'textvariable', 'image', 'bitmap', 
                            'compound', 'padx', 'pady', 'width', 'height'):
            if kw.get(set_size_if) is not None:
                W, H = self._info_button(
                        text = self.cnf.get('text'), 
                        font = self.cnf.get('font'), 
                        image = self.cnf.get('image'),
                        bitmap = self.cnf.get('bitmap'),
                        padding = ( self.cnf.get('padx',0), self.cnf.get('pady',0) ), 
                        compound = self.cnf.get('compound'),
                        textvariable = self.cnf.get('textvariable') )
                W = kw.get('width', W)
                H = kw.get('height', H)
                self._configure('configure', { 'width': W, 'height': H })
                break

        if self.cnf.get('image'):
            if kw.get('activeimage') is not None:
                self.bind_class('active_img','<Enter>', lambda _:self.itemconfig('_img',image=kw['activeimage'])   )
                self.bind_class('active_img','<Leave>', lambda _:self.itemconfig('_img',image=self.cnf.get('image'))   )
            elif kw.get('activeimage') is '':
                self.unbind_class('active_img','<Enter>')
                self.unbind_class('active_img','<Leave>')
                self.itemconfig('_img',image=self.cnf.get('image'))
            # For Image config
            config_cnf_img = {}
            for i in ('anchor', 'image'):
                config_cnf_img.update( {i : kw.get(i, self.cnf.get(i))} )
            self.itemconfig('_img', config_cnf_img, state=self['state'])
        elif self.cnf.get('bitmap'):
            if kw.get('activebitmap') is not None:
                self.bind_class('active_bit','<Enter>', lambda _:self.itemconfig('_bit',image=kw['activebitmap'])   )
                self.bind_class('active_bit','<Leave>', lambda _:self.itemconfig('_bit',image=self.cnf.get('bitmap'))   )
            elif kw.get('activebitmap') is '':
                self.unbind_class('active_bit','<Enter>')
                self.unbind_class('active_bit','<Leave>')
                self.itemconfig('_bit',image=self.cnf.get('bitmap'))
            # For bitmap config
            config_cnf_bit = {}
            for i in ('anchor', 'bitmap'):
                config_cnf_bit.update( {i : kw.get(i, self.cnf.get(i))} )
            self.itemconfig('_bit', config_cnf_bit, state=self['state'])

        if kw.get('compound'):
            self._compound(kw.get('compound'), self.winfo_reqheight(), self.winfo_reqwidth())
        
        if self.cnf.get('textvariable') is not None:
            self.cnf['text'] = self.cnf['textvariable'].get()
            self.cnf['textvariable'].trace_add( 'write', lambda *ag: 
                    self.itemconfig('_txt', text = self.cnf['textvariable'].get() ))

        # For Text config
        config_cnf_txt = {}
        for i in ('text','anchor','font','justify'):
            config_cnf_txt.update( {    i : kw.get(i, self.cnf.get(i))  } )
        self.itemconfig( '_txt', config_cnf_txt, state=self['state'], 
                        fill=kw.get('fg', self.cnf.get('fg')), 
                        disabledfill=kw.get('disabledforeground', 
                        self.cnf.get('disabledforeground')))  
        # if commented and state='disabled then doesnt disable completely. (NEED FIX)
        self.after(10, lambda: self.itemconfig( '_txt', state=self['state']))   
             
        if int(self['takefocus']) and self['state'] == 'normal':
            self.bind_class('takefocus', '<FocusIn>' , lambda _: self.itemconfig('_tf', state='normal'))
            self.bind_class('takefocus', '<FocusOut>', lambda _: self.itemconfig('_tf', state='hidden'))
        elif not int(self['takefocus']) or self['state'] == 'disabled':
            self.unbind_class('takefocus', '<FocusIn>' )
            self.unbind_class('takefocus', '<FocusOut>')
            self.itemconfig('_tf', state='hidden')
    
        Edge_color = get_shade(self['bg'], 0.1, 'auto-120')    # This will darken the border around the button
        self.itemconfig('_border1', outline=Edge_color)
        self.itemconfig('_border2', fill=Edge_color)

        if kw.get('bd'):
            self.itemconfig('_bd_color1', width=kw.get('bd', 6))
            self.itemconfig('_bd_color2', width=kw.get('bd', 6))

        if bool(kw.get('borderless')): 
            # Modify configurations of master widget to support `borderless=1`.
            def configure(cnf=None, **kw):
                """Configure resources of a widget.

                The values for resources are specified as keyword
                arguments. To get an overview about
                the allowed keyword arguments call the method keys.
                """
                #  Need a better fix ..
                kw = _TK._cnfmerge((cnf, kw))
                r = self.master._configure('configure', None, kw)
                if kw.get('bg') or kw.get('background'):
                    for i in self._buttons:
                        if i['borderless']:
                            i.cnf.update( {'bordercolor': i.master['bg']} )
                            i.itemconfig('_bd_color1', outline=i.master['bg'])
                            i.itemconfig('_bd_color2', fill=i.master['bg'])
                return r

            self.master.configure = configure
            self.master.config = configure
            self.cnf.update( {'bordercolor': self.master['bg']} )
            self.itemconfig('_bd_color1', outline=self.master['bg'])
            self.itemconfig('_bd_color2', fill=self.master['bg'])
        elif not bool(kw.get('borderless', True)) or not self.cnf.get('borderless'):
            if self.cnf.get('bordercolor') == self.master['bg']:
                self.cnf.pop('bordercolor', None)
            bd_color = self.cnf.get('bordercolor', get_shade(self['bg'], 0.04, 'auto-120'))
            self.cnf.update( {'bordercolor': bd_color} )
            self.itemconfig('_bd_color1', outline=kw.get('bordercolor', bd_color) )
            self.itemconfig('_bd_color2', fill=kw.get('bordercolor', bd_color) )
    
    def bind_class(self, className, sequence=None, func=None, add='+'):
        className = className+str(self)
        bindtags = list( self.bindtags() )
        if className in bindtags: bindtags.remove(className)
        bindtags.insert(bindtags.index('Canvas'), className)
        self.bindtags(bindtags)
        return super().bind_class(className, sequence=sequence, func=func, add=add)
    
    def unbind_class(self, className, sequence):
        className = className+str(self)
        bindtags = list(self.bindtags())
        if className in bindtags: bindtags.remove(className)
        self.bindtags(bindtags)
        return super().unbind_class(className, sequence)

    def on_press_color(self, **kw):
        '''### Give gradient color effect
        Internal funtion. return tag
        #### Arguments:
        1. `color`: ("#4b91fe", "#055be5")
        2. `tag`
        3. `height`
        4. `width`'''
        self.cnf['activebackground'] = self.cnf.get('activebackground', ("#4b91fe", "#055be5"))
        width = self.coords('_activebg')
        if len(width) > 3:
            width = width[2]
        if self.winfo_height() == len(self.find('withtag', '_activebg')) \
            and self.winfo_width() == width and not kw.get('color'): return
        self.delete('_activebg')
        tag = kw.get('tag', 'press')
        if kw.get('color') is None: kw.pop('color', None)
        cr = kw.get( 'color', ("#4b91fe", "#055be5") )  # This is the default color for mac 
        if isinstance(cr, tuple):
            if None in cr: 
                cr = list(cr)
                cr.remove(None)
                cr = cr[0]
        w, h = self.winfo_width(), self.winfo_height()
        if not isinstance(cr, tuple): cr = ( C.Color(cr),C.Color(cr) )
        else: cr = ( C.Color(cr[0]),C.Color(cr[1]) )
        for i, j in enumerate(list( cr[0].range_to( cr[1],  kw.get('heigh', h)) )):
            self._line(0, i, kw.get('width',w), i, fill=j, tag=tag, state='hidden')
        self.tag_lower(tag)     # keep the tag last 
        return tag 

    def _info_button(self, **kw):
        """Internal Funtion.\n
        This function takes essentials parameters to give
        the approximate width and height accordingly. \n
        It creates a ttk button and use all the resouces given 
        and returns width and height of the ttk button, after taking 
        width and height the button gets destroyed also the custom style."""
        _style_tmp = ttk.Style()
        _style_tmp.configure('%s.TButton'%self, font=kw.get('font') ) 
        _style_tmp.configure('%s.TButton'%self, padding=kw.get('padding') )
        tmp = ttk.Button(self, text=kw.get('text'), image=kw.get('image'), 
            bitmap=kw.get('bitmap'), textvariable=kw.get('textvariable'),
            compound=kw.get('compound'), style='%s.TButton'%self )
        geo = tmp.winfo_reqwidth(), tmp.winfo_reqheight()
        del _style_tmp   # Need fix --- doesn't really delete the custom style
        tmp.destroy()
        return geo

    def _compound(self, flag, height, width):
        "Internal function. Use `compound = 'left'/'right'/'top'/'bottom'` to configure."
        _PiTag = '_img' if self.cnf.get('image') else '_bit'
        _im_size = self.bbox(_PiTag)
        _txt_size = self.bbox('_txt')
        if _im_size and _txt_size:
            W_im = _im_size[2] - _im_size[0]
            H_im = _im_size[3] - _im_size[1]
            W_txt = _txt_size[2] - _txt_size[0]
            H_txt = _txt_size[3] - _txt_size[1]
            if flag is 'bottom':
                width = (width/2, width/2)
                height = (height/2-H_im/2, height/2+H_txt/2)
            elif flag is 'top' :
                width = (width/2, width/2)
                height = (height/2+H_im/2, height/2-H_txt/2)
            elif flag is 'right' :
                width = (width/2-W_im/2, width/2+W_txt/2)
                height = (height/2, height/2)
            elif flag is'left':
                width = (width/2+W_im/2, width/2-W_txt/2)
                height = (height/2, height/2)
            elif flag is not None:
                raise _TK.TclError('bad compound "{}": must be \
                    none, text, image, center, top, bottom, left, or right'.format(flag))
            if isinstance(height, tuple):
                self.coords('_txt', width[0], height[0])
                self.coords(_PiTag, width[1], height[1])
                return { 'width':width, 'height':height }
        return None
    
    def keys(self):
        """Return a list of all resource names of this widget."""
        K_all = ['background', 'bd', 'bg', 'borderwidth', 'cursor', 'height', 'highlightbackground', 'highlightcolor', 
            'highlightthickness','relief', 'state', 'takefocus', 'width', 'activebackground', 'activeforeground', 
            'activeimage', 'activebitmap', 'anchor', 'bitmap', 'command', 'compound', 'disabledforeground', 'fg', 
            'font', 'foreground', 'image', 'overrelief', 'padx', 'pady', 'repeatdelay', 'repeatinterval', 'text', 
            'textvariable', 'underline', 'bordercolor', 'borderless', 'disabledbackground']
        K_all.sort()
        return K_all



#                       Copyright 2020 Saad Mairaj
# 
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
# 
#        http://www.apache.org/licenses/LICENSE-2.0
# 
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import os
import tkinter as tk
import tkinter.ttk as ttk
from tkinter.colorchooser import askcolor
from PIL import Image, ImageTk
from tkmacosx.widget import *
from tkmacosx.variables import *
from tkmacosx.colorscale import Colorscale
from tkmacosx.colors import Hex as C_dict

def grid(root,row,column):
    "Defines rows and columns if grid method is used"
    if column:
        for y in range(column): tk.Grid.columnconfigure(root, y, weight=1)
    if row:
        for x in range(row): tk.Grid.rowconfigure(root, x, weight=1)
    return

class sample(tk.Tk):
    def __init__(self):
        super(sample, self).__init__()
        self.resizable(0,0)
        self.geometry('420x700+300+100')
        self.title('Mac OSX Button Testing')
        self.wm_attributes('-modified',1)
        self.main_color = ColorVar(value='lightpink')
        self['bg'] = self.main_color
        grid(self, 20, 5)
        self.L1 = tk.Label(self, text='Comparison', bg=self.main_color, font=('',18,'bold'))
        self.L1.grid(row=0, column=0, columnspan=5, sticky='nsew')
        Button(self, text='Hello').grid(row=1, column=1, sticky='s')
        ttk.Button(self, text='Hello').grid(row=1, column=3, sticky='s')
        tk.Button(self, text='Hello').grid(row=1, column=2, sticky='s')
        tk.Label(self, bg=self.main_color, font=('', 10),text='(Mac OSX)').grid(row=2, column=1, sticky='n',)
        tk.Label(self, bg=self.main_color, font=('', 10),text='(ttk themed)').grid(row=2, column=3, sticky='n')
        tk.Label(self, bg=self.main_color, font=('', 10),text='(Default)').grid(row=2, column=2, sticky='n')
        ttk.Separator(self, orient='vertical').grid(row=3, column=0, columnspan=5, sticky='ew')

        # ------------ Seperator -------------

        # ------------ Demonstration ------------

        self.sfr = SFrame(self, bg=self.main_color)
        self.sfr.grid(rowspan=16, columnspan=5, sticky='nsew')
        for i in range(5):
            self.sfr.grid_columnconfigure(i, weight=1)
        self.L2 = tk.Label(self.sfr, text='Demonstration', bg=self.main_color, font=('',20,'bold'))
        self.L2.grid(row=1, column=0, columnspan=5, sticky='new', pady=(20,10))

        # ------------ Active Color ------------
        
        self.L3 = tk.Label(self.sfr, text='1. Change Active color', bg=self.main_color, 
            font=('',15,'bold'))
        self.L3.grid(row=2, column=0, columnspan=5, sticky='nsew', pady=10)
        self.L4  =tk.Label(self.sfr, text='The active color can be changed to any gradient color.', 
            bg=self.main_color, font=('',10))
        self.L4.grid(row=3, column=0, columnspan=5, sticky='new')
        self.B1 = Button(self.sfr, text='Press Me', pady=20)
        self.B1.grid(row=4, column=0, columnspan=5, pady=20)
        self.C1 = tk.StringVar(value='Select')
        self.L5 = tk.Label(self.sfr, text='From', bg=self.main_color, font=('',12))
        self.L5.grid(row=5, column=1, sticky='nwe')
        self.Om1 = tk.OptionMenu(self.sfr, self.C1, *C_dict.keys(), 
            command=self.change_active_color)
        self.Om1.config(bg=self.main_color, width=15)
        self.Om1.grid(row=6, column=1,sticky='s', pady=(0,10))
        for i in range(self.Om1['menu'].index('end')+1):
            self.Om1['menu'].entryconfig(i, foreground=list(C_dict)[i])
        self.C2 = tk.StringVar(value='Select')
        self.L6 = tk.Label(self.sfr, text='To', bg=self.main_color, font=('',12))
        self.L6.grid(row=5, column=3, sticky='nwe')
        self.Om2 = tk.OptionMenu(self.sfr, self.C2, *C_dict.keys(), 
            command=self.change_active_color)
        self.Om2.config(bg=self.main_color, width=15)
        self.Om2.grid(row=6, column=3, sticky='s', pady=(0,10))
        for i in range(self.Om2['menu'].index('end')+1):
            self.Om2['menu'].entryconfig(i, foreground=list(C_dict)[i])
        

        # ------------ Backgroung Color ------------

        # ttk.Separator(self.sfr, orient='vertical').grid(row=6, column=2, columnspan=1, sticky='ew')
        self.L7 = tk.Label(self.sfr, text='2. Change Background color', bg=self.main_color, 
            font=('',15,'bold'))
        self.L7.grid(row=7, column=0, columnspan=5, sticky='nsew', pady=(50,0))
        self.L8  =tk.Label(self.sfr, text='Click on the button to choose the color.', 
            bg=self.main_color, font=('',10))
        self.L8.grid(row=8, column=0, columnspan=5, sticky='new', pady=10)

        self.B2 = Button(self.sfr, text='Color me', font=('',30,), pady=10, padx=10)
        self.B2.grid(row=9, column=0, columnspan=5, sticky='', pady=20)

        self.B3 = Button(self.sfr, text='Change Background Color', bg='#d0c0ea', borderless=1)
        self.B3['command'] = lambda: self.B2.config(bg=askcolor()[1])
        self.B3.grid(row=10, column=0, columnspan=5, sticky='w', pady=10)
        self.B4 = Button(self.sfr, text='Change Foreground Color', bg="#d0c0ea", borderless=1)
        self.B4['command'] = lambda: self.B2.config(fg=askcolor()[1])
        self.B4.grid(row=10, column=0, columnspan=5, sticky='e', pady=10)

        # ------------ Borderless ------------

        self.L9 = tk.Label(self.sfr, text='3. Switch Between Borderless', bg=self.main_color, 
            font=('',15,'bold'))
        self.L9.grid(row=11, column=0, columnspan=5, sticky='sew', pady=(50,0))
        self.L10 = tk.Label(self.sfr, text="""
    In borderless it will blend with its parent widget background color.
    Give parameter `borderless = True / False` to use it.""", bg=self.main_color, font=('',10))
        self.L10.grid(row=12, column=0, columnspan=5, sticky='new')

        self.B5 = Button(self.sfr, text='No Borders', borderless=1, height=40,
            bg='#212F3D', fg='white', activebackground=("#EAECEE", "#212F3D"))
        self.B5.grid(row=13, columnspan=5, pady=(20,5))

        self.B6 = Button(self.sfr, text='No Borders', borderless=1, height=40,
            bg='#F7DC6F', fg='#21618C', activebackground=('#B3B6B7','#58D68D'))
        self.B6.grid(row=14, columnspan=5, pady=(0,20))
        self.var1 = tk.BooleanVar(value=True)
        self.CB1 = tk.Checkbutton(self.sfr, text='Toggle Borderless',variable=self.var1, 
            bg=self.main_color, command=self.change_borderless_state)
        self.CB1.grid(row=15, columnspan=5, pady=(0,10))


        # ------------ Bordercolor ------------

        self.L11 = tk.Label(self.sfr, text='4. Change Bordercolor', bg=self.main_color, 
            font=('',15,'bold'))
        self.L11.grid(row=16, column=0, columnspan=5, sticky='sew', pady=(50,0))
        self.L12 = tk.Label(self.sfr, text="Change Bordercolor of the button\nNote: if borderless=True, then the bordercolor won't work.",
            bg=self.main_color, font=('',10))
        self.L12.grid(row=17, column=0, columnspan=5, sticky='new')

        self.B7 = Button(self.sfr, text='Border', pady=10, padx=5)
        self.B7.grid(row=18, columnspan=5, pady=30)

        self.B8 = Button(self.sfr, text='Change Bordercolor', borderless=1, bg="#d0c0ea") 
        self.B8['command']=lambda: self.B7.config(bordercolor=askcolor()[1])
        # self.B8.grid(row=19, columnspan=5)

        self.CS1 = Colorscale(self.sfr, value='hex', mousewheel=0,
            command=lambda e: self.B7.config(bordercolor=e))
        self.CS1.grid(row=19, columnspan=5, sticky='ew', padx=10)


        # ------------ Active Image and transparent Image ------------

        self.L11 = tk.Label(self.sfr, text='5. Active and Transparent Image', bg=self.main_color, 
            font=('',15,'bold'))
        self.L11.grid(row=20, column=0, columnspan=5, sticky='sew', pady=(50,0))
        self.L12 = tk.Label(self.sfr, text="Can pass activeimage to the button. Also it supports transparent images.",
            bg=self.main_color, font=('',10))
        self.L12.grid(row=21, column=0, columnspan=5, sticky='new')

        # from PIL import Image
        self.Img1 = Image.open(os.path.abspath(os.path.dirname(os.path.abspath(__file__))+'/images/B1.png'))
        self.Img_1 = ImageTk.PhotoImage(self.Img1.resize((200, 200), Image.ANTIALIAS))
        self.Img2 = Image.open(os.path.abspath(os.path.dirname(os.path.abspath(__file__))+'/images/B2.png'))
        self.Img_2 = ImageTk.PhotoImage(self.Img2.resize((200, 200), Image.ANTIALIAS))

        self.B10 = Button(self.sfr, image=self.Img_1, activeimage=self.Img_2, bg='#D7BDE2', 
            activebackground=('lightyellow','lightblue'), borderless=1)
        self.B10.grid(row=22, columnspan=5, pady=20)

        self.Scale1 = tk.Scale(self.sfr, orient='vertical', from_=100, to=300, bg=self.main_color, bd=0,
            command=lambda v: self.on_size_scale('vertical', v), troughcolor='#505452', font=('',10,'bold'),
            length=400, showvalue=0, width=10, sliderlength=60, resolution=50 )
        self.Scale1.bind('<Enter>', lambda _: self.Scale1.config(showvalue=1))
        self.Scale1.bind('<Leave>', lambda _: self.Scale1.config(showvalue=0))
        self.Scale1.grid(row=22, columnspan=5, sticky='w')
        self.Scale1.set(200)
        self.update_idletasks()

    def change_active_color(self, *ags):
        c1 = self.C1.get() if not self.C1.get() == 'Select' else None
        c2 = self.C2.get() if not self.C2.get() == 'Select' else None
        self.Om1.config(bg=c1)
        self.Om2.config(bg=c2)
        self.B1['activebackground'] = (c1, c2)
    
    def change_borderless_state(self):
        if self.var1.get(): 
            self.B5['borderless'] = 1
            self.B6['borderless'] = 1
        else: 
            self.B5['borderless'] = 0
            self.B6['borderless'] = 0

    def on_size_scale(self, scale, value):
        value = int(float(value))
        if scale == 'vertical':
            self.Img_1 = ImageTk.PhotoImage(self.Img1.resize((value, value), Image.ANTIALIAS))
            self.Img_2 = ImageTk.PhotoImage(self.Img2.resize((value, value), Image.ANTIALIAS))
            self.B10['image'] = self.Img_1
            self.B10['activeimage'] = self.Img_2
        
        
#  Testing Demo 
if __name__ == "__main__":
    sample().mainloop()



#                       Copyright 2020 Saad Mairaj
# 
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
# 
#        http://www.apache.org/licenses/LICENSE-2.0
# 
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

'''
Newer style colorchoosers for tkinter module.

Version: 0.1.0
'''

import tkinter as tk
from tkmacosx.basewidget import _Canvas
from PIL import Image, ImageTk
from tkinter.font import Font
from tkinter import font
import numpy as np
import os


class Colorscale(_Canvas):
    """
    ## Color Scale.
    This is ColorScale alternate to tkinter's colorchooser. 

    ### Args: 
    - `value`: Get either 'RGB' or 'HEX'.
    - `command`: callback function with an argument.
    - `orient`: Set the orientation.
    - `mousewheel`: Set mousewheel to scroll marker.
    - `variable`: Give tkinter variable (`StringVar`).
    - `showinfo`: Shows hex or rbg while selecting color.
    - `showinfodelay`: Delay before the showinfo disappears (in ms).
    """

    def __init__(self, master=None, cnf={}, **kw):
        kw = tk._cnfmerge((cnf, kw))
        self.cnf = {
            'value'         :    kw.pop('value', 'rgb'),
            'command'       :    kw.pop("command", None),
            'orient'        :    kw.pop('orient', 'vertical'),
            'mousewheel'    :    kw.pop("mousewheel", True),
            'variable'      :    kw.pop('variable', None),
            'showinfo'      :    kw.pop('showinfo', True),
            'showinfodelay' :    kw.pop('showinfodelay', 1500),
        }
        kw['width'] = kw.get("width",  250 if 'ver' in self.cnf['orient'] else 30)
        kw['height'] = kw.get("height", 30 if 'ver' in self.cnf['orient'] else 250)
        kw['highlightthickness'] = kw.get('highlightthickness', 0)
        tk.Canvas.__init__(self, master=master, **kw)
        self.xy_axis = int(self.winfo_width()/3)
        self._size = (0,0)

        self.np_im = np.load(os.path.dirname
                (os.path.abspath(__file__))+"/images/colorscale.npy")
        self.image_id = self._image(0, 0, anchor='nw')

        # Binds
        self.bind_class(self, '<Button-1>', self.move_marker, '+')
        self.bind_class(self, "<B1-Motion>", self.move_marker, '+')
        self.set_mousewheel()
        self.bind_class(self, '<Configure>', self.on_resize, '+')
    
    def set_mousewheel(self):
        '''Intenal function. Sets mousewheel scrolling.'''
        def on_mousewheel(evt=None):
            "Internal function."
            if evt.delta <= -1 and bool(self.xy_axis < self.winfo_width() \
                or self.xy_axis < self.winfo_height()):
                self.xy_axis += 1
                self.move_marker(evt, mw=self.xy_axis)
            if evt.delta >= 1 and self.xy_axis > 1:
                self.xy_axis -= 1
                self.move_marker(evt, mw=self.xy_axis)

        if self.cnf['mousewheel']:
            self.bind("<MouseWheel>", on_mousewheel)
        else: self.unbind("<MouseWheel>")

    def on_resize(self, evt):
        '''Internal function.'''
        if evt.width == self._size[0] and evt.height == self._size[1]: return
        self._size = (evt.width, evt.height)
        self._im = Image.fromarray(self.np_im).rotate(0 if 'ver' in
                            self.cnf['orient'] else 270, expand=1).resize(
                            (evt.width, evt.height))
        self.pixels = self._im.load()
        self._im = ImageTk.PhotoImage(self._im)
        self.itemconfig(self.image_id, image=self._im)
        
        self.delete('marker')
        self.delete('borderline1')
        self.delete('borderline2')
        self.rounded_rect(1, 1, evt.width-2, evt.height-2, 1, width=2, 
            fill='#81b3f4', tag1='borderline1', tag2='borderline2')
        self.rounded_rect(
            evt.width/3 if 'ver' in self.cnf['orient'] else 2,
            2 if 'ver' in self.cnf['orient'] else evt.height/3,
            5 if 'ver' in self.cnf['orient'] else evt.width-4,
            evt.height-4 if 'ver' in self.cnf['orient'] else 5,
            2, width=2, fill="black", tag="marker")
        
    def configure(self, cnf={}, **kw):
        """Configure resources of a widget.

        The values for resources are specified as keyword
        arguments. To get an overview about
        the allowed keyword arguments call the method keys.
        """
        cnf = tk._cnfmerge((cnf, kw))
        for k in cnf.copy():
            if k in self.cnf.keys():
                self.cnf[k] = cnf.pop(k)
        self.set_mousewheel()
        rv = super().configure(**cnf) 
        if rv is not None:
            rv.update(self.cnf)
            return rv
    config = configure

    def cget(self, key):
        """Return the resource value for a KEY given as string."""
        if key in self.cnf.keys():
            return self.cnf[key]
        return super().cget(key)
    __getitem__=cget

    def Release(self, evt=None):
        "Internal function."
        self.delete("info")
        self.delete("markerbg")

    def RGB2HEX(self, R, G, B):
        "Internal function. Use to convert RGB to HEX"
        return '#%02x%02x%02x' % (R, G, B)

    def move_marker(self, evt, mw=None):
        "Internal function."
        if mw:
            evt.x = mw if 'ver' in self.cnf['orient'] else 10
            evt.y = 10 if 'ver' in self.cnf['orient'] else mw

        self.after_cancel(getattr(self, '_remove_id', ' '))
        self._remove_id = self.after(self.cnf['showinfodelay'], self.Release)

        if (evt.x > 0 and evt.y > 0 and evt.x < self.winfo_width() and evt.y < self.winfo_height()):
            if not mw:
                self.xy_axis = evt.x
            RGB = self.pixels[evt.x, evt.y][:-1]
            hexcode = self.RGB2HEX(RGB[0], RGB[1], RGB[2])
            maker_color = 'black' if (
                RGB[0]*0.299 + RGB[1]*0.587 + RGB[2]*0.114) > 110 else 'white'
            
            self.itemconfig('borderline1', outline=hexcode)
            self.itemconfig('borderline2', fill=hexcode)

            if self.cnf['value'] == "rgb":
                spacer = 50
                spacbg = 40
                text = str(RGB)
                self.callback(RGB)
            elif self.cnf['value'] == "hex":
                spacer = 35
                spacbg = 25
                text = hexcode
                self.callback(hexcode)

            self.delete("marker")
            self.Release()

            self.rounded_rect(
                evt.x if 'ver' in self.cnf['orient'] else 2,
                2 if 'ver' in self.cnf['orient'] else evt.y,
                5 if 'ver' in self.cnf['orient'] else self.winfo_width()-4,
                self.winfo_height()-4 if 'ver' in self.cnf['orient'] else 5,
                2, width=2, fill=maker_color, tag="marker")

            if bool(evt.x < self.winfo_width()-100 or evt.y < self.winfo_height()-100) and self['showinfo']:
                self._rounded(
                    evt.x+spacer-spacbg if 'ver' in self.cnf['orient'] else self.winfo_width()/2-6,
                    self.winfo_height()/2-6 if 'ver' in self.cnf['orient'] else evt.y+spacer-spacbg,
                    evt.x+spacer+spacbg if 'ver' in self.cnf['orient'] else self.winfo_width()/2+7,
                    self.winfo_height()/2+7 if 'ver' in self.cnf['orient'] else evt.y+spacer+spacbg,
                    6, fill=maker_color, tag="markerbg")
                self._text(
                    evt.x+spacer if 'ver' in self.cnf['orient'] else self.winfo_width()/2,
                    self.winfo_height()/2 if 'ver' in self.cnf['orient'] else evt.y+spacer,
                    angle=0 if 'ver' in self.cnf['orient'] else 90,
                    text=text, font=Font(size=10), tag="info", fill=hexcode)
            elif self['showinfo']:
                self._rounded(
                    evt.x-spacer-spacbg if 'ver' in self.cnf['orient'] else self.winfo_width()/2-6,
                    self.winfo_height()/2-6 if 'ver' in self.cnf['orient'] else evt.y-spacer-spacbg,
                    evt.x-spacer+spacbg if 'ver' in self.cnf['orient'] else self.winfo_width()/2+7,
                    self.winfo_height()/2+7 if 'ver' in self.cnf['orient'] else evt.y-spacer+spacbg,
                    6, fill=maker_color, tag="markerbg")
                self._text(
                    evt.x-spacer if 'ver' in self.cnf['orient'] else self.winfo_width()/2,
                    self.winfo_height()/2 if 'ver' in self.cnf['orient'] else evt.y-spacer,
                    angle=0 if 'ver' in self.cnf['orient'] else 90,
                    text=text, font=Font(size=10), tag="info", fill=hexcode)

    def callback(self, val):
        "Internal function."
        if self.cnf['variable']:
            self.cnf['variable'].set(val)
        if self.cnf['command']:
            return self.cnf['command'](val)

    def keys(self):
        """Return a list of all resource names of this widget."""
        return list(self.config().keys())


# ------------------------------------ #
#           Testing and demo           #
# ------------------------------------ #

def demo_colorscale():
    root = tk.Tk()
    root.config({"bg": "black"})
    root.title("Tkinter Color Bar")
    CS = Colorscale(root, value='hex')
    CS.pack(side="bottom", padx=3, pady=3)
    root.mainloop()
    return "break"

if __name__ == "__main__":
    demo_colorscale()


from setuptools import setup


def long_description():
    with open('Information.md') as f:
        longdis = f.read()
    return longdis


setup(
    name="tkmacosx",
    version="0.1.1",
    description="Tkmacosx is a Python library extension to the Tkinter module for macOS",
    long_description=long_description(),
    long_description_content_type="text/markdown",
    url="https://github.com/Saadmairaj/tkmacosx",
    author="Saad Mairaj",
    author_email="Saadmairaj@yahoo.in",
    license="Apache",
    classifiers=[
        "Intended Audience :: Developers",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Build Tools",
        'Topic :: Software Development :: Widget Sets',
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Environment :: MacOS X",

    ],
    keywords=['tkinter', 'macos', 'variable', 'widgets', 'colorscale', 'tk'],
    packages=["tkmacosx"],
    include_package_data=True,
    install_requires=['colour', 'pillow'],
    project_urls={  # Optional
        'Bug Reports': 'https://github.com/Saadmairaj/tkmacosx',
        'Source': 'https://github.com/Saadmairaj/tkmacosx/issues',
    },
)
